<?php

namespace Ulmus\Api\Gitea\Entity;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\{Api\Attribute\Property\Entity,
    Attribute\Property\Field,
    Entity\Field\Datetime,
    EntityTrait,
    SearchRequest\SearchRequestInterface,
    SearchRequest\SearchRequestPaginationTrait};
use Ulmus\Api\Attribute\Obj\Api;

#[Api(adapter: "gitea")]
#[Api\Create(url: "/repos/{owner}/{id}")]
#[Api\Collection(url: "/repos/{owner}/{id}")]
#[Api\Read(url: "/repos/{owner}/{id}")]
#[Api\Update(url: "/repos/{owner}/{id}")]
#[Api\Delete(url: "/repos/{owner}/{id}")]
class Repository
{
    use EntityTrait;

    #[Field\Id]
    public int $id;

    #[Field]
    public string $title;

    #[Field]
    public string $body;

    #[Field]
    public int $number;

    #[Field(name: "html_url")]
    public string $htmlUrl;

    #[Field(name:"original_url")]
    public ? string $originalUrl;

    #[Field(name: "is_locked")]
    public bool $isLocked;

    #[Field]
    public array $labels = [];

    #[Field\CreatedAt(name: "created_at")]
    public ? Datetime $createdAt;

    #[Field\UpdatedAt(name: "updated_at")]
    public ? Datetime $updatedAt;

    #[Entity]
    public User $owner;

    public static function searchRequest(...$arguments) : SearchRequestInterface
    {
        return new class($arguments[0]) implements SearchRequestInterface
        {
            use SearchRequestPaginationTrait;

            public ? string $file = null;

            public string $createdAt = "ASC";

            public function __construct(ServerRequestInterface $request)
            {
                $get = new \ArrayObject(array_filter($request->getQueryParams(), function($i) { return $i !== ""; }));

                $this->page = $get->offsetExists('page') ? $get['page'] : 1;
                $this->file = $get->offsetExists('file') ? $get['file'] : null;
                $this->createdAt = $get->offsetExists('created_at') ? $get['created_at'] : "DESC";
            }

            public function filter(\Ulmus\Repository $repository): \Ulmus\Repository {
                return $repository;
            }

            public function wheres() : iterable
            {
                return array_filter([
                ], function($i) { return ! is_null($i); });
            }

            public function likes(): iterable
            {
                return array_filter([
                ], function($i) { return ! is_null($i) && $i !== ""; });
            }

            public function groups(): iterable
            {
                return [];
            }

            public function orders(): iterable
            {
                return array_filter([
                ], function($i) { return ! is_null($i) && $i !== ""; });
            }
        };
    }
}